import inspect
from typing import List, Optional, Union

from ldap3.core.exceptions import LDAPException

from perun.connector.adapters.AdapterInterface import AdapterInterface
from perun.connector.adapters.LdapAdapter import (
    AdapterSkipException,
    LdapAdapter,
    LDAPNotExistsException,
)
from perun.connector.adapters.PerunRpcAdapter import (
    PerunRpcAdapter,
    RPCAdapterNotExistsException,
)
from perun.connector.models.Facility import Facility
from perun.connector.models.Group import Group
from perun.connector.models.Member import Member
from perun.connector.models.Resource import Resource
from perun.connector.models.User import User
from perun.connector.models.UserExtSource import UserExtSource
from perun.connector.models.VO import VO
from perun.connector.perun_openapi import ApiException
from perun.connector.perun_openapi.exceptions import NotFoundException
from perun.connector.utils.Logger import Logger


class AdaptersManagerException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class AdaptersManagerNotExistsException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class AdaptersManager(AdapterInterface):
    def __init__(self, config, attrs_map):
        self._logger = Logger.get_logger(self.__class__.__name__)
        self._STARTING_PRIORITY = 1
        self.adapters = {}

        adapters_info = config["adapters"]

        for adapter_info in adapters_info:
            config_data = adapter_info.copy()
            adapter_type = config_data.pop("type")
            priority = config_data.pop("priority")

            if adapter_type == "ldap":
                ldap_adapter = LdapAdapter(config_data, attrs_map)

                self.adapters[priority] = {
                    "name": "ldap_adapter",
                    "adapter": ldap_adapter,
                }
            elif adapter_type == "openApi":
                rpc_adapter = PerunRpcAdapter(config_data, attrs_map)

                self.adapters[priority] = {
                    "name": "rpc_adapter",
                    "adapter": rpc_adapter,
                }
            else:
                self._logger.warning(
                    f'Config file includes unsupported adapter type "'
                    f'{adapter_type}"'
                )

    def _execute_method_by_priority(self, method_name: str, *args):
        current_priority = self._STARTING_PRIORITY
        current_adapter = self.adapters.get(current_priority)

        while current_adapter is not None:
            adapter_impl = current_adapter["adapter"]
            try:
                return getattr(adapter_impl, method_name)(*args)
            except AdapterSkipException as e:
                self._logger.warning(
                    f'{e.message} Method: "{method_name}" Adapter: '
                    f'{current_adapter["name"]} Going to try another '
                    f"adapter if available."
                )
                current_priority += 1
                current_adapter = self.adapters.get(current_priority)
            except (ApiException, LDAPException) as ex:
                if (ex.body and "NotExistsException" in ex.body) or isinstance(
                    ex, NotFoundException
                ):
                    raise AdaptersManagerNotExistsException(ex.body)
                self._logger.warning(
                    f'Method "{method_name}" could not be executed '
                    f'successfully by {current_adapter["name"]}, exception '
                    f'occurred: "{ex}" Going to try another '
                    f"adapter if available."
                )
                current_priority += 1
                current_adapter = self.adapters.get(current_priority)
            except (LDAPNotExistsException, RPCAdapterNotExistsException) as ex:
                raise AdaptersManagerNotExistsException(ex.body)

        raise AdaptersManagerException(
            f'None of the provided adapters was able to resolve method "'
            f'{method_name}"'
        )

    def _get_caller_name(self):
        return inspect.stack()[1].function

    def get_perun_user(self, idp_id: str, uids: List[str]) -> Optional[User]:
        return self._execute_method_by_priority(self._get_caller_name(), idp_id, uids)

    def get_group_by_name(self, vo: Union[int, VO], name: str) -> Group:
        return self._execute_method_by_priority(self._get_caller_name(), vo, name)

    def get_vo(self, short_name="", vo_id=None) -> VO:
        return self._execute_method_by_priority(
            self._get_caller_name(), short_name, vo_id
        )

    def get_member_groups(
        self, user: Union[int, User], vo: Union[int, VO]
    ) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), user, vo)

    def get_sp_groups_by_facility(self, facility: Union[Facility, int]) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), facility)

    def get_sp_groups_by_rp_id(self, rp_id: str) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), rp_id)

    def get_resource_attributes(
        self, resource: Union[int, Resource], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        return self._execute_method_by_priority(
            self._get_caller_name(), resource, attr_names
        )

    def get_user_attributes(
        self, user: Union[int, User], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, attr_names
        )

    def get_entityless_attribute(
        self, attr_name: str
    ) -> Union[str, Optional[int], bool, List[str], dict[str, str]]:
        return self._execute_method_by_priority(self._get_caller_name(), attr_name)

    def get_vo_attributes(
        self, vo: Union[int, VO], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        return self._execute_method_by_priority(self._get_caller_name(), vo, attr_names)

    def get_facility_by_rp_identifier(self, rp_identifier: str) -> Facility:
        return self._execute_method_by_priority(self._get_caller_name(), rp_identifier)

    def get_users_groups_on_facility_by_rp_id(
        self, rp_identifier: str, user: Union[User, int]
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(), rp_identifier, user
        )

    def get_users_groups_on_facility(
        self, facility: Union[Facility, int], user: Union[User, int]
    ) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), facility, user)

    def get_facilities_by_attribute_value(
        self, attribute: dict[str, str]
    ) -> List[Facility]:
        return self._execute_method_by_priority(self._get_caller_name(), attribute)

    def get_facility_attributes(
        self, facility: Union[int, Facility], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        return self._execute_method_by_priority(
            self._get_caller_name(), facility, attr_names
        )

    def get_user_ext_source(
        self, ext_source_name: str, ext_source_login: str
    ) -> UserExtSource:
        return self._execute_method_by_priority(
            self._get_caller_name(), ext_source_name, ext_source_login
        )

    def get_user_ext_source_by_unique_attribute_value(
        self,
        attr_name: str,
        attr_value: Union[str, int, bool, List[str], dict[str, str]],
    ) -> UserExtSource:
        return self._execute_method_by_priority(
            self._get_caller_name(), attr_name, attr_value
        )

    def update_user_ext_source_last_access(self, user_ext_source: str) -> None:
        return self._execute_method_by_priority(
            self._get_caller_name(), user_ext_source
        )

    def get_user_ext_source_attributes(
        self, user_ext_source: Union[int, UserExtSource], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        return self._execute_method_by_priority(
            self._get_caller_name(), user_ext_source, attr_names
        )

    def set_user_ext_source_attributes(
        self,
        user_ext_source: Union[int, UserExtSource],
        attributes: dict[
            str, Union[str, Optional[int], bool, List[str], dict[str, str]]
        ],
    ) -> None:
        return self._execute_method_by_priority(
            self._get_caller_name(), user_ext_source, attributes
        )

    def set_user_attributes(
        self,
        user: Union[User, int],
        attributes: dict[
            str, Union[str, Optional[int], bool, List[str], dict[str, str]]
        ],
    ) -> None:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, attributes
        )

    def get_member_status_by_user_and_vo(
        self, user: Union[int, User], vo: Union[int, VO]
    ) -> str:
        return self._execute_method_by_priority(self._get_caller_name(), user, vo)

    def is_user_in_vo_by_short_name(
        self, user: Union[int, User], vo_short_name: str
    ) -> bool:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, vo_short_name
        )

    def get_resource_capabilities_by_facility(
        self, facility: Union[Facility, int], user_groups: List[Union[Group, int]]
    ) -> List[str]:
        return self._execute_method_by_priority(
            self._get_caller_name(), facility, user_groups
        )

    def get_resource_capabilities_by_rp_id(
        self, rp_identifier: str, user_groups: List[Union[Group, int]]
    ) -> List[str]:
        return self._execute_method_by_priority(
            self._get_caller_name(), rp_identifier, user_groups
        )

    def get_facility_capabilities_by_rp_id(self, rp_identifier: str) -> List[str]:
        return self._execute_method_by_priority(self._get_caller_name(), rp_identifier)

    def get_facility_capabilities_by_facility(
        self, facility: Union[Facility, int]
    ) -> List[str]:
        return self._execute_method_by_priority(self._get_caller_name(), facility)

    def get_groups_where_member_is_active(
        self, member: Union[Member, int]
    ) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), member)

    def get_groups_where_user_as_member_is_active(
        self, user: Union[User, int], vo: Union[VO, int]
    ) -> List[Group]:
        return self._execute_method_by_priority(self._get_caller_name(), user, vo)

    def has_registration_form_group(self, group: Union[Group, int]) -> bool:
        return self._execute_method_by_priority(self._get_caller_name(), group)

    def has_registration_form_vo(self, vo: Union[VO, int]) -> bool:
        return self._execute_method_by_priority(self._get_caller_name(), vo)

    def has_registration_form_by_vo_short_name(self, vo_short_name: str) -> bool:
        return self._execute_method_by_priority(self._get_caller_name(), vo_short_name)

    def create_facility(self, name: str, description="") -> Facility:
        return self._execute_method_by_priority(
            self._get_caller_name(), name, description
        )

    def set_facility_attributes(
        self,
        facility: Union[Facility, int],
        attributes: dict[
            str, Union[str, Optional[int], bool, List[str], dict[str, str]]
        ],
    ) -> None:
        return self._execute_method_by_priority(
            self._get_caller_name(), facility, attributes
        )

    def get_attributes_definition(self) -> List[dict[str, Union[str, int, bool]]]:
        return self._execute_method_by_priority(self._get_caller_name())

    def get_member_by_user(
        self, user: Union[User, int], vo: Union[VO, int]
    ) -> Optional[Member]:
        return self._execute_method_by_priority(self._get_caller_name(), user, vo)

    def is_user_perun_admin(self, user: Union[User, int]) -> bool:
        return self._execute_method_by_priority(self._get_caller_name(), user)

    def get_vos_where_user_is_admin(
        self, user: Union[User, int], check_perun_admin=True
    ) -> List[VO]:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, check_perun_admin
        )

    def get_groups_where_user_is_admin(
        self,
        user: Union[User, int],
        check_perun_admin=True,
        fill_group_unique_name=True,
        fetch_related_vos=True,
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(),
            user,
            check_perun_admin,
            fill_group_unique_name,
            fetch_related_vos,
        )

    def get_facilities_where_user_is_admin(
        self, user: Union[User, int], check_perun_admin=True, fill_facility_rp_id=True
    ) -> List[Facility]:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, check_perun_admin, fill_facility_rp_id
        )

    def get_resources_where_user_is_admin(
        self,
        user: Union[User, int],
        check_perun_admin=True,
        fetch_related_vos=True,
        fetch_related_facilities=True,
        fill_facility_rp_id=True,
    ) -> List[Resource]:
        return self._execute_method_by_priority(
            self._get_caller_name(),
            user,
            check_perun_admin,
            fetch_related_vos,
            fetch_related_facilities,
            fill_facility_rp_id,
        )

    def get_all_vos(self) -> List[VO]:
        return self._execute_method_by_priority(self._get_caller_name())

    def get_all_facilities(self, fill_facility_rp_id=True) -> List[Facility]:
        return self._execute_method_by_priority(
            self._get_caller_name(), fill_facility_rp_id
        )

    def get_all_groups(
        self, fetch_related_vos=True, fill_group_unique_name=True
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(), fetch_related_vos, fill_group_unique_name
        )

    def get_all_resources(
        self,
        fetch_related_vos=True,
        fetch_related_facilities=True,
        fill_facility_rp_id=True,
    ) -> List[Resource]:
        return self._execute_method_by_priority(
            self._get_caller_name(),
            fetch_related_vos,
            fetch_related_facilities,
            fill_facility_rp_id,
        )

    def get_groups_where_user_is_active_resource(
        self,
        user: Union[User, int],
        resource: Union[Resource, int],
        map_unique_name=True,
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(), user, resource, map_unique_name
        )

    def get_facility_by_id(self, facility_id: int) -> Facility:
        return self._execute_method_by_priority(self._get_caller_name(), facility_id)

    def get_resources_for_facility(
        self, facility: Union[Facility, int], map_vo=True
    ) -> List[Resource]:
        return self._execute_method_by_priority(
            self._get_caller_name(), facility, map_vo
        )

    def get_groups_for_resource(
        self, resource: Union[Resource, int], map_unique_name=True
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(), resource, map_unique_name
        )

    def get_resource_by_id(
        self, resource_id: int, map_vo=True, map_facility=True
    ) -> Resource:
        return self._execute_method_by_priority(
            self._get_caller_name(), resource_id, map_vo, map_facility
        )

    def get_all_parent_groups_for_group(
        self, group: Union[Group, int], map_unique_name=True
    ) -> List[Group]:
        return self._execute_method_by_priority(
            self._get_caller_name(), group, map_unique_name
        )

    def get_facilities_by_attribute_with_attributes(
        self,
        attr_name: str,
        attr_value: Union[str, int, bool, List[str], dict[str, str]],
        attr_names: List[str],
    ) -> List[Facility]:
        return self._execute_method_by_priority(
            self._get_caller_name(), attr_name, attr_value, attr_names
        )



from unittest.mock import MagicMock, patch

import pytest

from perun.connector.adapters.LdapAdapter import LdapAdapter
from perun.connector.models.Facility import Facility
from perun.connector.models.Group import Group
from perun.connector.models.MemberStatusEnum import MemberStatusEnum
from perun.connector.models.User import User
from perun.connector.models.VO import VO
from perun.connector.utils.ConfigStore import ConfigStore

adapters_cfg = ConfigStore.get_adapters_manager_config().get("adapters")
ldapAdapterCfg = None
for adapter in adapters_cfg:
    if adapter["type"] == "ldap":
        ldapAdapterCfg = adapter
        break

ADAPTER = LdapAdapter(ldapAdapterCfg, ConfigStore.get_attribute_map())

# USER
USER_WITH_ALL = {
    "perunUserId": 1,
    "displayName": "Foe Toe",
    "cn": "Foe Toe",
    "mail": "foetoe@cesnet.cz",
}
USER_WITH_DN = {"perunUserId": 1, "displayName": "Foe Toe", "cn": []}
USER_WITH_CN = {"perunUserId": 1, "displayName": "", "cn": ["Foe Toe"]}
USER_WITHOUT_NAME = {"perunUserId": 1, "displayName": "", "cn": []}
USER_NOT_FOUND = None
USER = User(1, "Foe Toe")
USER_WITHOUT_ID = User(None, "Foe Without Toe")
USER_WITHOUT_NAME_INIT = User(1, None)
USER_DATA = {
    "perunUserId": 1,
    "displayName": "Foe Toe",
    "cn": "Foe Toe",
    "memberOf": [
        "perunGroupId=1,perunVoId=1,dc=perun,dc=cesnet,dc=cz",
        "perunGroupId=2,perunVoId=1,dc=perun,dc=cesnet,dc=cz",
    ],
}
USER_DATA_EMPTY = {
    "perunUserId": 1,
    "displayName": "Foe Toe",
    "cn": "Foe Toe",
    "memberOf": [],
}

# VO
TEST_VO = VO(1, "organization", "org")
TEST_FALSE_VO = VO(2, "fake", "fk")
VO_1 = {"perunVoId": "1", "description": ["voo1"], "o": ["vo1"]}
VO_2 = {"perunVoId": "1", "description": ["organization"], "o": ["org"]}

INITIALIZED_VO_1 = VO(1, "voo1", "vo1")
INITIALIZED_VO_2 = VO(1, "organization", "org")

# GROUP
GROUP_1 = {
    "perunGroupId": 1,
    "perunVoId": 1,
    "uuid": "",
    "cn": ["group1"],
    "perunUniqueGroupName": "grp1",
    "description": ["this group1"],
}
GROUP_2 = {
    "perunGroupId": 2,
    "perunVoId": 1,
    "uuid": "",
    "cn": ["group2"],
    "perunUniqueGroupName": "grp2",
    "description": ["this group2"],
}
GROUP_3 = {
    "perunGroupId": 3,
    "perunVoId": 1,
    "uuid": "",
    "cn": ["group3"],
    "perunUniqueGroupName": "grp3",
    "description": ["this group3"],
}
INITIALIZED_GROUP_1 = Group(1, TEST_VO, "", "group1", "grp1", "this group1")
INITIALIZED_GROUP_2 = Group(2, TEST_VO, "", "group2", "grp2", "this group2")
INITIALIZED_GROUP_3 = Group(3, TEST_VO, "", "group3", "grp3", "this group3")
INITIALIZED_GROUPS = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2, INITIALIZED_GROUP_3]
GROUPS_REPEATED = [GROUP_1, GROUP_1, GROUP_2, GROUP_3]
GROUPS = [GROUP_1, GROUP_2, GROUP_3]
GROUPS_EMPTY = []
GROUP_ID = {"perunGroupId": "1"}

# FACILITY
FACILITY_DATA = {
    "perunFacilityId": "1",
    "cn": "facility",
    "description": "this is a testing facility",
    "capabilities": ["capability1", "capability2"],
}
FACILITY = Facility(1, "facility", "this is a testing facility", "1", {})
FACILITY_EMPTY = None

# RESOURCE
RESOURCE_1 = {
    "perunVoId": "1",
    "assignedGroupId": ["1"],
    "perunResourceId": "1",
    "capabilities": ["capability1, capability2"],
}
RESOURCE_2 = {
    "perunVoId": "1",
    "assignedGroupId": ["1", "2", "3"],
    "perunResourceId": "2",
    "capabilities": ["capability1, capability2"],
}
RESOURCE_3 = {
    "perunVoId": "1",
    "assignedGroupId": [],
    "perunResourceId": "3",
    "capabilities": ["capability3"],
}
RESOURCES_REPEATED = [RESOURCE_1, RESOURCE_2]
RESOURCES_NOT_GROUPS = [RESOURCE_3]
RESOURCES_EMPTY = []
RESOURCES = [RESOURCE_2]


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_user_with_all(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=USER_WITH_ALL)
    user = ADAPTER.get_perun_user("1", ["1"])
    assert user == USER


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_user_with_dn(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=USER_WITH_DN)
    user = ADAPTER.get_perun_user("1", ["1"])
    assert user == USER


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_user_with_cn(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=USER_WITH_CN)
    user = ADAPTER.get_perun_user("1", ["1"])
    assert user == USER


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_user_without_name(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=USER_WITHOUT_NAME)
    user = ADAPTER.get_perun_user("1", ["1"])
    assert user == USER_WITHOUT_NAME_INIT


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_user_not_found(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=USER_NOT_FOUND)
    user = ADAPTER.get_perun_user("1", ["1"])
    assert user is None


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_group_exist_in_vo(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(side_effect=(GROUP_1, VO_2))
    group = ADAPTER.get_group_by_name(TEST_VO, "grp")
    assert group == INITIALIZED_GROUP_1


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_group_does_not_exist_in_vo(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=None)

    expected_error_message = (
        "Group with name: grp in VO: " "2 does not exists in Perun LDAP."
    )

    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_group_by_name(TEST_FALSE_VO, "grp")

        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_vo_by_short_name_that_exists(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=VO_1)
    vo = ADAPTER.get_vo("vo1")
    assert vo == INITIALIZED_VO_1


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_vo_by_id_that_exist(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=VO_1)
    vo = ADAPTER.get_vo(None, 1)
    assert vo == INITIALIZED_VO_1


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_vo_by_non_existent_short_name(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=None)
    expected_error_message = (
        "Vo with name: fake_short_name " "does not exists in Perun LDAP."
    )

    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_vo("fake_short_name")

        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_vo_by_non_existent_id(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=None)
    expected_error_message = "Vo with id: -1 does " "not exists in Perun LDAP."

    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_vo(None, -1)

        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_member_groups(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(
        side_effect=[USER_DATA, GROUP_1, VO_2, GROUP_2, VO_2]
    )
    groups = ADAPTER.get_member_groups(USER, TEST_VO)

    expected_groups = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2]
    assert expected_groups.sort(key=lambda x: x.id) == groups.sort(key=lambda x: x.id)


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_member_groups_empty(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(
        side_effect=[USER_DATA_EMPTY, GROUP_1, GROUP_2]
    )
    groups = ADAPTER.get_member_groups(USER, TEST_VO)
    assert groups == []


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_sp_groups(mock_request, mock_request2, mock_request3):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES)
    ADAPTER.connector.search_for_entity = MagicMock(
        side_effect=[GROUP_1, GROUP_2, GROUP_3]
    )

    groups = ADAPTER.get_sp_groups_by_facility(FACILITY)
    expected_groups = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2, INITIALIZED_GROUP_3]
    assert expected_groups.sort(key=lambda x: x.id) == groups.sort(key=lambda x: x.id)


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_sp_groups_repeated_groups(mock_request, mock_request2, mock_request3):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_REPEATED)
    ADAPTER.connector.search_for_entity = MagicMock(
        side_effect=[GROUP_1, GROUP_1, GROUP_2, GROUP_3]
    )

    groups = ADAPTER.get_sp_groups_by_facility(FACILITY)
    expected_groups = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2, INITIALIZED_GROUP_3]
    assert expected_groups.sort(key=lambda x: x.id) == groups.sort(key=lambda x: x.id)


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_get_sp_groups_not_assigned_groups(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_NOT_GROUPS)

    groups = ADAPTER.get_sp_groups_by_facility(FACILITY)
    assert groups == []


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_get_sp_groups_empty_resources(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_EMPTY)

    expected_error_message = (
        "Service with spEntityId: " + str(1) + " hasn't assigned any resource."
    )
    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_sp_groups_by_facility(FACILITY)
        assert str(error.value.args[0]) == expected_error_message


# TODO UPDATE AFTER FILLING LDAP IN ATTRIBUTES CONFIG
"""
@patch(
    "perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier"
)
def test_user_attributes(mock_request):
    empty_user_attributes = []
    ADAPTER.connector.search_for_entity = MagicMock(
        side_effect=[USER_WITH_ALL, USER_NOT_FOUND]
    )
    attributes = ADAPTER.get_user_attributes(USER, empty_user_attributes)
    assert attributes['displayName'] == 'Foe Toe'
    assert attributes['cn'] == 'Foe Toe'
    assert attributes['mail'] == 'foetoe@cesnet.cz'
    assert attributes['perunUserId'] == 1

    attributes = ADAPTER.get_user_attributes(USER, ['fakeAttribute'])
    assert not attributes
"""


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_get_facility_by_rp_id(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=FACILITY_DATA)
    facility = ADAPTER.get_facility_by_rp_identifier("1")
    assert facility == FACILITY


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
def test_users_group_on_facility_facility_not_found(mock_request):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY_EMPTY)
    groups = ADAPTER.get_users_groups_on_facility(None, None)
    assert groups == []


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_users_group_on_facility_resources_not_found(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_EMPTY)
    expected_error_message = (
        "Service with spEntityId: " "1 hasn't assigned any resource."
    )

    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_users_groups_on_facility(FACILITY, USER)

        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_users_group_on_facility_groups_not_found(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(
        side_effect=[RESOURCES_REPEATED, GROUPS_EMPTY]
    )
    groups = ADAPTER.get_users_groups_on_facility(FACILITY, USER)

    assert groups == []


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_users_group_on_facility_repeated_groups(mock_request, mock_request2):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.connector.search_for_entities = MagicMock(
        side_effect=[RESOURCES_REPEATED, GROUPS_REPEATED]
    )
    groups = ADAPTER.get_users_groups_on_facility(FACILITY, USER)
    expected_groups = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2, INITIALIZED_GROUP_3]
    assert expected_groups.sort(key=lambda x: x.id) == groups.sort(key=lambda x: x.id)


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_users_group_on_facility_not_repeated_groups(mock_request, mock_request2):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.connector.search_for_entities = MagicMock(
        side_effect=[RESOURCES_REPEATED, GROUPS]
    )
    groups = ADAPTER.get_users_groups_on_facility(FACILITY, USER)
    expected_groups = [INITIALIZED_GROUP_1, INITIALIZED_GROUP_2, INITIALIZED_GROUP_3]
    assert expected_groups.sort(key=lambda x: x.id) == groups.sort(key=lambda x: x.id)


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_member_status_invalid(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=None)
    expected_error_message = (
        "Member status is other than valid. "
        + "Skipping to another adapter to get MemberStatus"
    )
    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_member_status_by_user_and_vo(USER, TEST_VO)
        assert str(error.value.args[0] == expected_error_message)


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_member_status_valid(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=GROUP_ID)
    validity = ADAPTER.get_member_status_by_user_and_vo(USER, TEST_VO)
    assert validity == MemberStatusEnum.VALID


def test_is_user_in_vo_no_short_name():
    expected_error_message = "voShortName is empty"

    with pytest.raises(Exception) as error:
        _ = ADAPTER.is_user_in_vo_by_short_name(USER, "")

        assert str(error.value.args[0]) == expected_error_message


def test_is_user_in_vo_user_without_id():
    expected_error_message = "userId is empty"

    with pytest.raises(Exception) as error:
        _ = ADAPTER.is_user_in_vo_by_short_name(USER_WITHOUT_ID, "org")

        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch(
    "perun.connector.adapters.LdapAdapter.LdapAdapter.get_member_status_by_user_and_vo"
)
def test_is_not_user_in_vo(mock_request, mock_request2):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.get_member_status_by_user_and_vo = MagicMock(
        return_value=MemberStatusEnum.INVALID
    )
    user_in_vo = ADAPTER.is_user_in_vo_by_short_name(USER, "org")
    assert not user_in_vo


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_vo")
@patch(
    "perun.connector.adapters.LdapAdapter.LdapAdapter.get_member_status_by_user_and_vo"
)
def test_is_user_in_vo(mock_request, mock_request2):
    ADAPTER.get_vo = MagicMock(return_value=TEST_VO)
    ADAPTER.get_member_status_by_user_and_vo = MagicMock(
        return_value=MemberStatusEnum.VALID
    )
    user_in_vo = ADAPTER.is_user_in_vo_by_short_name(USER, "org")
    assert user_in_vo


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
def test_resource_capabilities_no_facility(mock_request):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY_EMPTY)
    resource_capabilities = ADAPTER.get_resource_capabilities_by_facility(
        FACILITY_EMPTY, INITIALIZED_GROUPS
    )
    assert resource_capabilities == []


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_resource_capabilities_empty(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_EMPTY)
    expected_error_message = (
        "Service with spEntityId: " + str(1) + " hasn't assigned any resource."
    )
    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_resource_capabilities_by_facility(FACILITY, INITIALIZED_GROUPS)
        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.adapters.LdapAdapter.LdapAdapter.get_facility_by_rp_identifier")
@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entities")
def test_resource_capabilities(mock_request, mock_request2):
    ADAPTER.get_facility_by_rp_identifier = MagicMock(return_value=FACILITY)
    ADAPTER.connector.search_for_entities = MagicMock(return_value=RESOURCES_REPEATED)
    resource_capabilities = ADAPTER.get_resource_capabilities_by_facility(
        FACILITY, INITIALIZED_GROUPS
    )

    assert len(resource_capabilities) == 2
    assert "capability1, capability2" in resource_capabilities


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_facility_capabilities_empty(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=None)
    expected_error_message = "Facility with id: " + str(1) + " not found."
    with pytest.raises(Exception) as error:
        _ = ADAPTER.get_facility_capabilities_by_facility(FACILITY)
        assert str(error.value.args[0]) == expected_error_message


@patch("perun.connector.connectors.LdapConnector.LdapConnector.search_for_entity")
def test_facility_capabilities(mock_request):
    ADAPTER.connector.search_for_entity = MagicMock(return_value=FACILITY_DATA)
    facility_capabilities = ADAPTER.get_facility_capabilities_by_facility(FACILITY)
    assert facility_capabilities == FACILITY_DATA["capabilities"]



"""
    Perun RPC API

    Perun Remote Procedure Calls Application Programming Interface  # noqa: E501

    The version of the OpenAPI document: 0.0.0
    Contact: perun@cesnet.cz
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from perun.connector.perun_openapi.api_client import ApiClient, Endpoint as _Endpoint
from perun.connector.perun_openapi.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types,
)
from perun.connector.perun_openapi.model.facility import Facility
from perun.connector.perun_openapi.model.input_get_facilities import InputGetFacilities
from perun.connector.perun_openapi.model.input_get_match_resources import (
    InputGetMatchResources,
)
from perun.connector.perun_openapi.model.input_get_members_by_user_attributes import (
    InputGetMembersByUserAttributes,
)
from perun.connector.perun_openapi.model.input_get_resources import InputGetResources
from perun.connector.perun_openapi.model.input_get_users import InputGetUsers
from perun.connector.perun_openapi.model.member import Member
from perun.connector.perun_openapi.model.perun_exception import PerunException
from perun.connector.perun_openapi.model.resource import Resource
from perun.connector.perun_openapi.model.user import User


class SearcherApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.get_attributes_match_resources_endpoint = _Endpoint(
            settings={
                "response_type": ([Resource],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/searcher/getResources/attributes-match",
                "operation_id": "get_attributes_match_resources",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "input_get_match_resources",
                ],
                "required": [
                    "input_get_match_resources",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "input_get_match_resources": (InputGetMatchResources,),
                },
                "attribute_map": {},
                "location_map": {
                    "input_get_match_resources": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.get_attributes_resources_endpoint = _Endpoint(
            settings={
                "response_type": ([Resource],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/searcher/getResources/attributes",
                "operation_id": "get_attributes_resources",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "input_get_resources",
                ],
                "required": [
                    "input_get_resources",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "input_get_resources": (InputGetResources,),
                },
                "attribute_map": {},
                "location_map": {
                    "input_get_resources": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.get_facilities_endpoint = _Endpoint(
            settings={
                "response_type": ([Facility],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/searcher/getFacilities",
                "operation_id": "get_facilities",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "input_get_facilities",
                ],
                "required": [
                    "input_get_facilities",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "input_get_facilities": (InputGetFacilities,),
                },
                "attribute_map": {},
                "location_map": {
                    "input_get_facilities": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.get_members_by_user_attributes_endpoint = _Endpoint(
            settings={
                "response_type": ([Member],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/searcher/getMembersByUserAttributes",
                "operation_id": "get_members_by_user_attributes",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "input_get_members_by_user_attributes",
                ],
                "required": [
                    "input_get_members_by_user_attributes",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "input_get_members_by_user_attributes": (
                        InputGetMembersByUserAttributes,
                    ),
                },
                "attribute_map": {},
                "location_map": {
                    "input_get_members_by_user_attributes": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.get_users_searcher_endpoint = _Endpoint(
            settings={
                "response_type": ([User],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/searcher/getUsers",
                "operation_id": "get_users_searcher",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "input_get_users",
                ],
                "required": [
                    "input_get_users",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "input_get_users": (InputGetUsers,),
                },
                "attribute_map": {},
                "location_map": {
                    "input_get_users": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )

    def get_attributes_match_resources(self, input_get_match_resources, **kwargs):
        """Get list of resources that have attributes with partially matched values if allowPartialMatchForString is set to true, else with exactly matched values.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_attributes_match_resources(input_get_match_resources, async_req=True)
        >>> result = thread.get()

        Args:
            input_get_match_resources (InputGetMatchResources):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Resource]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["input_get_match_resources"] = input_get_match_resources
        return self.get_attributes_match_resources_endpoint.call_with_http_info(
            **kwargs
        )

    def get_attributes_resources(self, input_get_resources, **kwargs):
        """Get list of resources who have attributes with specific values.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_attributes_resources(input_get_resources, async_req=True)
        >>> result = thread.get()

        Args:
            input_get_resources (InputGetResources):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Resource]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["input_get_resources"] = input_get_resources
        return self.get_attributes_resources_endpoint.call_with_http_info(**kwargs)

    def get_facilities(self, input_get_facilities, **kwargs):
        """Get list of facilities who have attributes with specific values.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_facilities(input_get_facilities, async_req=True)
        >>> result = thread.get()

        Args:
            input_get_facilities (InputGetFacilities):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Facility]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["input_get_facilities"] = input_get_facilities
        return self.get_facilities_endpoint.call_with_http_info(**kwargs)

    def get_members_by_user_attributes(
        self, input_get_members_by_user_attributes, **kwargs
    ):
        """Get list of members who have attributes with specific values.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_members_by_user_attributes(input_get_members_by_user_attributes, async_req=True)
        >>> result = thread.get()

        Args:
            input_get_members_by_user_attributes (InputGetMembersByUserAttributes):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Member]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs[
            "input_get_members_by_user_attributes"
        ] = input_get_members_by_user_attributes
        return self.get_members_by_user_attributes_endpoint.call_with_http_info(
            **kwargs
        )

    def get_users_searcher(self, input_get_users, **kwargs):
        """Get list of users who have attributes with specific values.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_users_searcher(input_get_users, async_req=True)
        >>> result = thread.get()

        Args:
            input_get_users (InputGetUsers):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [User]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["input_get_users"] = input_get_users
        return self.get_users_searcher_endpoint.call_with_http_info(**kwargs)



"""
    Perun RPC API

    Perun Remote Procedure Calls Application Programming Interface  # noqa: E501

    The version of the OpenAPI document: 0.0.0
    Contact: perun@cesnet.cz
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from perun.connector.perun_openapi.api_client import ApiClient, Endpoint as _Endpoint
from perun.connector.perun_openapi.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types,
)
from perun.connector.perun_openapi.model.facility_state import FacilityState
from perun.connector.perun_openapi.model.perun_exception import PerunException
from perun.connector.perun_openapi.model.resource_state import ResourceState
from perun.connector.perun_openapi.model.service_state import ServiceState
from perun.connector.perun_openapi.model.task import Task
from perun.connector.perun_openapi.model.task_and_destination_id_object import (
    TaskAndDestinationIdObject,
)
from perun.connector.perun_openapi.model.task_and_destination_name_object import (
    TaskAndDestinationNameObject,
)
from perun.connector.perun_openapi.model.task_id_object import TaskIdObject
from perun.connector.perun_openapi.model.task_result import TaskResult
from perun.connector.perun_openapi.model.task_result_id_object import TaskResultIdObject


class TasksManagerApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.count_tasks_endpoint = _Endpoint(
            settings={
                "response_type": (int,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/countTasks",
                "operation_id": "count_tasks",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [],
                "required": [],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {},
                "attribute_map": {},
                "location_map": {},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.delete_task_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/deleteTask",
                "operation_id": "delete_task",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_id_object",
                ],
                "required": [
                    "task_id_object",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_id_object": (TaskIdObject,),
                },
                "attribute_map": {},
                "location_map": {
                    "task_id_object": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.delete_task_result_by_id_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/deleteTaskResultById",
                "operation_id": "delete_task_result_by_id",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_result_id_object",
                ],
                "required": [
                    "task_result_id_object",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_result_id_object": (TaskResultIdObject,),
                },
                "attribute_map": {},
                "location_map": {
                    "task_result_id_object": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.delete_task_results_by_ids_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/urlinjsonout/tasksManager/deleteTaskResultsByIds",
                "operation_id": "delete_task_results_by_ids",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_result_ids",
                ],
                "required": [
                    "task_result_ids",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_result_ids": ([int],),
                },
                "attribute_map": {
                    "task_result_ids": "taskResultIds",
                },
                "location_map": {
                    "task_result_ids": "query",
                },
                "collection_format_map": {
                    "task_result_ids": "multi",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.delete_task_results_ids_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/deleteTaskResults/id",
                "operation_id": "delete_task_results_ids",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_and_destination_id_object",
                ],
                "required": [
                    "task_and_destination_id_object",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_and_destination_id_object": (TaskAndDestinationIdObject,),
                },
                "attribute_map": {},
                "location_map": {
                    "task_and_destination_id_object": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.delete_task_results_names_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/deleteTaskResults/name",
                "operation_id": "delete_task_results_names",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_and_destination_name_object",
                ],
                "required": [
                    "task_and_destination_name_object",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_and_destination_name_object": (TaskAndDestinationNameObject,),
                },
                "attribute_map": {},
                "location_map": {
                    "task_and_destination_name_object": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": ["application/json"],
            },
            api_client=api_client,
        )
        self.get_all_facilities_states_endpoint = _Endpoint(
            settings={
                "response_type": ([FacilityState],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getAllFacilitiesStates",
                "operation_id": "get_all_facilities_states",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "vo",
                ],
                "required": [],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "vo": (int,),
                },
                "attribute_map": {
                    "vo": "vo",
                },
                "location_map": {
                    "vo": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_all_resources_state_endpoint = _Endpoint(
            settings={
                "response_type": ([ResourceState],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getAllResourcesState",
                "operation_id": "get_all_resources_state",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "vo_id",
                ],
                "required": [
                    "vo_id",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "vo_id": (int,),
                },
                "attribute_map": {
                    "vo_id": "voId",
                },
                "location_map": {
                    "vo_id": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_facility_services_state_endpoint = _Endpoint(
            settings={
                "response_type": ([ServiceState],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getFacilityServicesState",
                "operation_id": "get_facility_services_state",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "facility",
                ],
                "required": [
                    "facility",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "facility": (int,),
                },
                "attribute_map": {
                    "facility": "facility",
                },
                "location_map": {
                    "facility": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_facility_state_endpoint = _Endpoint(
            settings={
                "response_type": (FacilityState,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getFacilityState",
                "operation_id": "get_facility_state",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "facility",
                ],
                "required": [
                    "facility",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "facility": (int,),
                },
                "attribute_map": {
                    "facility": "facility",
                },
                "location_map": {
                    "facility": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_endpoint = _Endpoint(
            settings={
                "response_type": (Task,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTask",
                "operation_id": "get_task",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "service",
                    "facility",
                ],
                "required": [
                    "service",
                    "facility",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "service": (int,),
                    "facility": (int,),
                },
                "attribute_map": {
                    "service": "service",
                    "facility": "facility",
                },
                "location_map": {
                    "service": "query",
                    "facility": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_by_id_endpoint = _Endpoint(
            settings={
                "response_type": (Task,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskById",
                "operation_id": "get_task_by_id",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "id",
                ],
                "required": [
                    "id",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "id": (int,),
                },
                "attribute_map": {
                    "id": "id",
                },
                "location_map": {
                    "id": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_result_by_id_endpoint = _Endpoint(
            settings={
                "response_type": (TaskResult,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultById",
                "operation_id": "get_task_result_by_id",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "task_result",
                ],
                "required": [
                    "task_result",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task_result": (int,),
                },
                "attribute_map": {
                    "task_result": "taskResult",
                },
                "location_map": {
                    "task_result": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResults",
                "operation_id": "get_task_results",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [],
                "required": [],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {},
                "attribute_map": {},
                "location_map": {},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_by_task_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultsByTask",
                "operation_id": "get_task_results_by_task",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "task",
                ],
                "required": [
                    "task",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task": (int,),
                },
                "attribute_map": {
                    "task": "task",
                },
                "location_map": {
                    "task": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_for_destinations_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultsForDestinations",
                "operation_id": "get_task_results_for_destinations",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "destinations",
                ],
                "required": [
                    "destinations",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "destinations": ([str],),
                },
                "attribute_map": {
                    "destinations": "destinations[]",
                },
                "location_map": {
                    "destinations": "query",
                },
                "collection_format_map": {
                    "destinations": "multi",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_for_guiby_task_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultsForGUIByTask",
                "operation_id": "get_task_results_for_guiby_task",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "task",
                ],
                "required": [
                    "task",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task": (int,),
                },
                "attribute_map": {
                    "task": "task",
                },
                "location_map": {
                    "task": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_for_guiby_task_and_destination_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultsForGUIByTaskAndDestination",
                "operation_id": "get_task_results_for_guiby_task_and_destination",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "task",
                    "destination",
                ],
                "required": [
                    "task",
                    "destination",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task": (int,),
                    "destination": (int,),
                },
                "attribute_map": {
                    "task": "task",
                    "destination": "destination",
                },
                "location_map": {
                    "task": "query",
                    "destination": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.get_task_results_for_guiby_task_only_newest_endpoint = _Endpoint(
            settings={
                "response_type": ([TaskResult],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/getTaskResultsForGUIByTaskOnlyNewest",
                "operation_id": "get_task_results_for_guiby_task_only_newest",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "task",
                ],
                "required": [
                    "task",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "task": (int,),
                },
                "attribute_map": {
                    "task": "task",
                },
                "location_map": {
                    "task": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.is_there_such_task_endpoint = _Endpoint(
            settings={
                "response_type": (int,),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/isThereSuchTask",
                "operation_id": "is_there_such_task",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "service",
                    "facility",
                ],
                "required": [
                    "service",
                    "facility",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "service": (int,),
                    "facility": (int,),
                },
                "attribute_map": {
                    "service": "service",
                    "facility": "facility",
                },
                "location_map": {
                    "service": "query",
                    "facility": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.list_all_tasks_endpoint = _Endpoint(
            settings={
                "response_type": ([Task],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/listAllTasks",
                "operation_id": "list_all_tasks",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [],
                "required": [],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {},
                "attribute_map": {},
                "location_map": {},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.list_all_tasks_for_facility_endpoint = _Endpoint(
            settings={
                "response_type": ([Task],),
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/listAllTasksForFacility",
                "operation_id": "list_all_tasks_for_facility",
                "http_method": "GET",
                "servers": None,
            },
            params_map={
                "all": [
                    "facility",
                ],
                "required": [
                    "facility",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "facility": (int,),
                },
                "attribute_map": {
                    "facility": "facility",
                },
                "location_map": {
                    "facility": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )
        self.suspend_tasks_propagation_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["BasicAuth", "BearerAuth"],
                "endpoint_path": "/json/tasksManager/suspendTasksPropagation",
                "operation_id": "suspend_tasks_propagation",
                "http_method": "POST",
                "servers": None,
            },
            params_map={
                "all": [
                    "suspend",
                ],
                "required": [
                    "suspend",
                ],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "suspend": (bool,),
                },
                "attribute_map": {
                    "suspend": "suspend",
                },
                "location_map": {
                    "suspend": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

    def count_tasks(self, **kwargs):
        """Returns the count of all tasks.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.count_tasks(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            int
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        return self.count_tasks_endpoint.call_with_http_info(**kwargs)

    def delete_task(self, task_id_object, **kwargs):
        """Delete Task and TaskResults.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_task(task_id_object, async_req=True)
        >>> result = thread.get()

        Args:
            task_id_object (TaskIdObject):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_id_object"] = task_id_object
        return self.delete_task_endpoint.call_with_http_info(**kwargs)

    def delete_task_result_by_id(self, task_result_id_object, **kwargs):
        """Delete TaskResult by its ID.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_task_result_by_id(task_result_id_object, async_req=True)
        >>> result = thread.get()

        Args:
            task_result_id_object (TaskResultIdObject):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_result_id_object"] = task_result_id_object
        return self.delete_task_result_by_id_endpoint.call_with_http_info(**kwargs)

    def delete_task_results_by_ids(self, task_result_ids, **kwargs):
        """Delete TaskResults by their IDs. TaskResults are expected to be from the same facility.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_task_results_by_ids(task_result_ids, async_req=True)
        >>> result = thread.get()

        Args:
            task_result_ids ([int]): list of TaskResult ids

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_result_ids"] = task_result_ids
        return self.delete_task_results_by_ids_endpoint.call_with_http_info(**kwargs)

    def delete_task_results_ids(self, task_and_destination_id_object, **kwargs):
        """Delete all TaskResults for specified Task and Destination.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_task_results_ids(task_and_destination_id_object, async_req=True)
        >>> result = thread.get()

        Args:
            task_and_destination_id_object (TaskAndDestinationIdObject):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_and_destination_id_object"] = task_and_destination_id_object
        return self.delete_task_results_ids_endpoint.call_with_http_info(**kwargs)

    def delete_task_results_names(self, task_and_destination_name_object, **kwargs):
        """Delete all TaskResults for specified Task and Destination.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_task_results_names(task_and_destination_name_object, async_req=True)
        >>> result = thread.get()

        Args:
            task_and_destination_name_object (TaskAndDestinationNameObject):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_and_destination_name_object"] = task_and_destination_name_object
        return self.delete_task_results_names_endpoint.call_with_http_info(**kwargs)

    def get_all_facilities_states(self, **kwargs):
        """Return propagation status of all facilities in Perun/Vo.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_all_facilities_states(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            vo (int): id of Vo. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [FacilityState]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        return self.get_all_facilities_states_endpoint.call_with_http_info(**kwargs)

    def get_all_resources_state(self, vo_id, **kwargs):
        """Return propagation status of all resources related to VO.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_all_resources_state(vo_id, async_req=True)
        >>> result = thread.get()

        Args:
            vo_id (int): VO id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [ResourceState]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["vo_id"] = vo_id
        return self.get_all_resources_state_endpoint.call_with_http_info(**kwargs)

    def get_facility_services_state(self, facility, **kwargs):
        """Returns service states for defined facility.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_facility_services_state(facility, async_req=True)
        >>> result = thread.get()

        Args:
            facility (int): id of Facility

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [ServiceState]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["facility"] = facility
        return self.get_facility_services_state_endpoint.call_with_http_info(**kwargs)

    def get_facility_state(self, facility, **kwargs):
        """Return propagation status of facility.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_facility_state(facility, async_req=True)
        >>> result = thread.get()

        Args:
            facility (int): id of Facility

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            FacilityState
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["facility"] = facility
        return self.get_facility_state_endpoint.call_with_http_info(**kwargs)

    def get_task(self, service, facility, **kwargs):
        """Returns a task.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task(service, facility, async_req=True)
        >>> result = thread.get()

        Args:
            service (int): id of Service
            facility (int): id of Facility

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            Task
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["service"] = service
        kwargs["facility"] = facility
        return self.get_task_endpoint.call_with_http_info(**kwargs)

    def get_task_by_id(self, id, **kwargs):
        """Returns Task by its id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_by_id(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): numeric id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            Task
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["id"] = id
        return self.get_task_by_id_endpoint.call_with_http_info(**kwargs)

    def get_task_result_by_id(self, task_result, **kwargs):
        """Returns TaskResult by its id.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_result_by_id(task_result, async_req=True)
        >>> result = thread.get()

        Args:
            task_result (int): TaskResult id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            TaskResult
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task_result"] = task_result
        return self.get_task_result_by_id_endpoint.call_with_http_info(**kwargs)

    def get_task_results(self, **kwargs):
        """Returns all task results.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        return self.get_task_results_endpoint.call_with_http_info(**kwargs)

    def get_task_results_by_task(self, task, **kwargs):
        """Return list of TaskResults by a Task.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results_by_task(task, async_req=True)
        >>> result = thread.get()

        Args:
            task (int): Task id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task"] = task
        return self.get_task_results_by_task_endpoint.call_with_http_info(**kwargs)

    def get_task_results_for_destinations(self, destinations, **kwargs):
        """Returns task results for defined destinations.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results_for_destinations(destinations, async_req=True)
        >>> result = thread.get()

        Args:
            destinations ([str]): List<String> Destinations

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["destinations"] = destinations
        return self.get_task_results_for_destinations_endpoint.call_with_http_info(
            **kwargs
        )

    def get_task_results_for_guiby_task(self, task, **kwargs):
        """Return list of TaskResults by a Task for GUI.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results_for_guiby_task(task, async_req=True)
        >>> result = thread.get()

        Args:
            task (int): Task id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task"] = task
        return self.get_task_results_for_guiby_task_endpoint.call_with_http_info(
            **kwargs
        )

    def get_task_results_for_guiby_task_and_destination(
        self, task, destination, **kwargs
    ):
        """Return list of TaskResults by a Task and destination for GUI.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results_for_guiby_task_and_destination(task, destination, async_req=True)
        >>> result = thread.get()

        Args:
            task (int): Task id
            destination (int): id of Destination

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task"] = task
        kwargs["destination"] = destination
        return self.get_task_results_for_guiby_task_and_destination_endpoint.call_with_http_info(
            **kwargs
        )

    def get_task_results_for_guiby_task_only_newest(self, task, **kwargs):
        """Return list of only newest TaskResults by a Task for GUI.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_task_results_for_guiby_task_only_newest(task, async_req=True)
        >>> result = thread.get()

        Args:
            task (int): Task id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [TaskResult]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["task"] = task
        return self.get_task_results_for_guiby_task_only_newest_endpoint.call_with_http_info(
            **kwargs
        )

    def is_there_such_task(self, service, facility, **kwargs):
        """Whether task exists.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.is_there_such_task(service, facility, async_req=True)
        >>> result = thread.get()

        Args:
            service (int): id of Service
            facility (int): id of Facility

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            int
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["service"] = service
        kwargs["facility"] = facility
        return self.is_there_such_task_endpoint.call_with_http_info(**kwargs)

    def list_all_tasks(self, **kwargs):
        """Returns all tasks.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_all_tasks(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Task]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        return self.list_all_tasks_endpoint.call_with_http_info(**kwargs)

    def list_all_tasks_for_facility(self, facility, **kwargs):
        """Returns all tasks associated with selected facility.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_all_tasks_for_facility(facility, async_req=True)
        >>> result = thread.get()

        Args:
            facility (int): id of Facility

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [Task]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["facility"] = facility
        return self.list_all_tasks_for_facility_endpoint.call_with_http_info(**kwargs)

    def suspend_tasks_propagation(self, suspend, **kwargs):
        """Suspends waiting tasks propagation to the engine. Does not affect already propagated tasks.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.suspend_tasks_propagation(suspend, async_req=True)
        >>> result = thread.get()

        Args:
            suspend (bool): if true stops propagating waiting tasks to the engine, if false resumes propagation

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs["async_req"] = kwargs.get("async_req", False)
        kwargs["_return_http_data_only"] = kwargs.get("_return_http_data_only", True)
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")
        kwargs["_request_auths"] = kwargs.get("_request_auths", None)
        kwargs["suspend"] = suspend
        return self.suspend_tasks_propagation_endpoint.call_with_http_info(**kwargs)



from perun.connector.models.HasIdAbstract import HasIdAbstract


class VO(HasIdAbstract):
    def __init__(self, id: int, name: str, short_name: str):
        super().__init__(id)
        self.name = name
        self.short_name = short_name

    def __str__(self):
        return f"id: {self.id} name: {self.name} short_name: {self.short_name}"

    def __eq__(self, other):
        if isinstance(other, VO):
            return (
                self.id == other.id
                and self.name == other.name
                and self.short_name == other.short_name
            )
        return False

    def __hash__(self):
        return hash((self.id, self.name, self.short_name))



from perun.connector.adapters.AdapterInterface import AdapterInterface
from perun.connector.adapters.AdaptersManager import AdaptersManager
from perun.connector.adapters.LdapAdapter import AdapterSkipException
from perun.connector.models.Facility import Facility
from perun.connector.models.Group import Group
from perun.connector.models.HasIdAbstract import HasIdAbstract
from perun.connector.models.Member import Member
from perun.connector.models.MemberStatusEnum import MemberStatusEnum
from perun.connector.models.Resource import Resource
from perun.connector.models.User import User
from perun.connector.models.UserExtSource import UserExtSource
from perun.connector.models.VO import VO
from perun.connector.utils.AttributeUtils import AttributeUtils
from perun.connector.utils.Logger import Logger

__all__ = [
    "AdapterInterface",
    "AdaptersManager",
    "AdapterSkipException",
    "Logger",
    "AttributeUtils",
    "VO",
    "User",
    "Group",
    "Member",
    "Facility",
    "Resource",
    "UserExtSource",
    "MemberStatusEnum",
    "HasIdAbstract",
]
